/** 
 * SALSA 2
 * July 2000
 * Mike Gasser (gasser@indiana.edu)
 */

import java.awt.*;
import java.awt.event.*;

abstract class CritterCanvas extends Canvas {
    
  protected final static Font
    FONT = new Font("SansSerif", Font.PLAIN, 10);
    
  public final static Color
    BACKGROUND_COLOR = Color.white,
    DEAD_COLOR = Color.lightGray,
    HIGHLIGHT_COLOR = Color.blue,
    SHOW_COLOR = Color.magenta,
    BORDER_COLOR = Color.black,
    BORDER_COLOR1 = Color.darkGray,
    BORDER_COLOR2 = Color.gray,
    BORDER_COLOR3 = Color.lightGray,
    BORDER_COLOR4 = Cell.LIGHTEST_GRAY;

  protected int
    highlightedCritter = 1000,
    nCritters,
    legendY0,
    inputWidth,
    outputWidth,
    namesWidth,
    mainWidth,
    width,
    height,
    mainHeight,
    legendHeight,
    cellWidth,
    cellHeight;
    
  protected UI ui;
  
  protected World world;
  
  protected Image image;

  protected Graphics ig;
  
  protected char[] names;

  protected boolean[]
    dead;
  
  CritterCanvas (UI ui, World world) {
      this.ui = ui;
      this.world = world;
      nCritters = world.getNCritters();
      names = getNames();
      init();
      setDimensions();
      dead = new boolean[nCritters];
      for (int i = 0; i < nCritters; i++)
        dead[i] = false;
      setBackground(BACKGROUND_COLOR);
  } // constructor CritterCanvas(UI, World)
  
  abstract protected void init();
  
  abstract protected void setDimensions();
  
  /**
  * Set the size of the canvas and the frame.
  */
  public Dimension getMinimumSize() {
    return new Dimension (width, height);
  } // getMinimumSize()

  public Dimension getPreferredSize() {
    return getMinimumSize();
  } // getPreferredSize()
  
  public int getCellHeight() {
      return cellHeight;
  } // getCellHeight()
    
  protected boolean isDead(int c) {
    return dead[c];
  } // isDead(int)

  protected char[] getNames() {
      char[] names = new char[world.getNCritters()];
      for (int critI = 0; critI < world.getNCritters(); critI++)
	  names[critI] = world.getCritter(critI).getSpeciesName();
      
      return names;
  } // getNames()

  protected Sensor[][] getSensors() {
      Sensor[][] sensors = new Sensor[world.getNCritters()][];
      
      for (int critI = 0; critI < world.getNCritters(); critI++)
	  sensors[critI] = world.getCritter(critI).getSensors();

      return sensors;
  } // getSensors()
  
  protected Effector[][] getEffectors() {
      Effector[][] effectors = new Effector[world.getNCritters()][];
      for (int critI = 0; critI < world.getNCritters(); critI++)
	  effectors[critI] = world.getCritter(critI).getEffectors();

      return effectors;
  } // getEffectors()

  protected Memory[] getMemories() {
      Memory[] memories = new Memory[world.getNCritters()];
      for (int critI = 0; critI < world.getNCritters(); critI++)
        memories[critI] = world.getCritter(critI).getMemory();
      
      return memories;
  } // getMemories()

  protected Strength[] getStrengths() {
      Strength[] strengths = new Strength[world.getNCritters()];
      for (int critI = 0; critI < world.getNCritters(); critI++)
        strengths[critI] = world.getCritter(critI).getStrength();
      
      return strengths;
  } // getStrengths()

  protected void initImage() {
    image = createImage(width, height);
    ig = image.getGraphics();

    drawLines(ig);
    initDrawCritters(ig);
    writeLegend(ig);
  }
  
  protected void initDrawCritters(Graphics g) {
    for (int c = 0; c < nCritters; c++) {
      ig.drawLine(0, (c + 1) * cellHeight, width - 1, (c + 1) * cellHeight);
      ig.drawString(String.valueOf(names[c]),
		    mainWidth + 3, c * cellHeight + cellHeight / 2 + 5);
    }
  }
  
  abstract protected void drawLines(Graphics g);
  
  abstract protected void writeLegend(Graphics g);
    
  abstract public void paintCritter(Graphics g, int critIndex);
  
  public void paint (Graphics g) {
    int critIndex;
    
    if (image == null)
	initImage();

    for (critIndex = 0; critIndex < nCritters; critIndex++)
	paintCritter(ig, critIndex);

    g.drawImage(image, 0, 0, this);

  } // paint(Graphics)
  
  public void highlightCritter(int critIndex, boolean on) {
      Graphics g = getGraphics();
      
      if (on)
        highlightedCritter = critIndex;
      else
        highlightedCritter = 1000;
      
      g.setColor (on ? HIGHLIGHT_COLOR : BORDER_COLOR);
      g.drawLine (0, critIndex * cellHeight, width, critIndex * cellHeight);
      g.drawLine (0, (critIndex + 1) * cellHeight,
        width, (critIndex + 1) * cellHeight);
  } // highlightCritter(int, boolean)
  
} // class CritterCanvas
