/** 
 * SALSA 2
 * July 2000
 * Mike Gasser (gasser@indiana.edu)
 */

import java.awt.*;
import java.io.*;
import java.awt.event.*;

class EditWorldFrame extends Frame
  implements ActionListener, WindowListener, ItemListener {  

  private static final String
    OK = "OK",
    CANCEL = "Cancel";

  public static final int
    NTYPES = 5,
    ROCK = 0,
    TENDER_PLANT = 1,
    TOUGH_PLANT = 2,
    FERTILE_CELL = 3,
    CRITTER = 4;
    
  private static final int
    X_OFFSET = 0,
    Y_OFFSET = 200;
    
  public static final Color[]
    SELECT_COLORS = {Rock.COLOR, Plant.COLOR2, Plant.COLOR,
                     Cell.FERTILITY_COLOR, Critter.COLOR};

  public static int nTypes;
      
  private Font f = new Font ("Helvetica", 0, 12);

  private World world;
  private WorldFrame frame;
  private UI ui;

  private Button
    okButton, cancelButton;
    
  private Choice typeChoice;
  
  private Panel choicePanel;
  
  private boolean
    rocks,
    plants,
    critters,
    fertileCells;
    
  EditWorldFrame(World world, UI ui)
  {
    this.ui = ui;
    this.world = world;
    this.frame = world.frame;
    
    nTypes = NTYPES - 1;

    choicePanel = new Panel();
    typeChoice = new Choice();
    typeChoice.setFont(f);
    typeChoice.addItem("Rocks");
    typeChoice.addItem("Tender plants");
    typeChoice.addItem("Tough plants");
    typeChoice.addItem("Fertile cells");
    for (int s = 0; s < Params.getNSpecies(); s++) {
      typeChoice.addItem("Critters " + Params.getSpecies(s).getName());
      nTypes++;
    }
  
    typeChoice.addItemListener(this);
    choicePanel.add(typeChoice);

    TextArea tA;
    tA = new TextArea("Select a thing type and click on cells to " +
  		      "\nadd or remove things from the world.",
			2, 35);
			  
    Panel bPanel = new Panel();
    bPanel.setLayout(new FlowLayout());
    okButton = new Button(OK);
    okButton.addActionListener(this);
    bPanel.add(okButton);
    cancelButton = new Button(CANCEL);
    cancelButton.addActionListener(this);
    bPanel.add(cancelButton);

    // Put everything together
    setLayout(new BorderLayout());
    add("North", tA);
    add("Center", choicePanel);
    add("South", bPanel);

    frame.initSelect();
    frame.select(Rock.COLOR, ROCK);
    addWindowListener(this);

    setTitle("Edit world");
      
    pack();
    setLocation(X_OFFSET, Y_OFFSET);
  
    setVisible (true);
  } // constructor EditWorldFrame

  public void windowClosed (WindowEvent event) {
  } // method windowClosed

  public void windowDeiconified (WindowEvent event) {
  } // method windowDeiconified

  public void windowIconified (WindowEvent event) {
  } // method windowIconified

  public void windowActivated (WindowEvent event) {
  } // method windowActivated

  public void windowDeactivated (WindowEvent event) {
  } // method windowDeactivated

  public void windowOpened (WindowEvent event) {
  } // method windowOpened

  public void windowClosing (WindowEvent event) {
    dispose();
  } // method windowClosing
  
  public void actionPerformed (ActionEvent event) {
      Object source = event.getSource();
      int c;
      
      if (source == okButton && frame.anySelectedCells()) {
 	    frame.deinitSelect();
	    dispose();
	    world.reinit(frame.getRemoveThings(), frame.getRemoveFertileCells(),
			 frame.getSelectedCells());
	    world.paint();

      } else {
	  frame.deinitSelect();
	  frame.repaint();
	  dispose();
      }
  } // method actionPerformed
  
  private Color getSelectColor(int i) {
      return SELECT_COLORS[Math.min(i, SELECT_COLORS.length - 1)];
  } // getSelectColor(int)

  public void itemStateChanged (ItemEvent event) {
    int i = typeChoice.getSelectedIndex();
    frame.select(getSelectColor(i), i);
  } // itemStateChanged(ItemEvent)

} // class EditWorldFrame
