/** 
 * SALSA 2
 * July 2000
 * Mike Gasser (gasser@indiana.edu)
 */

import java.io.*;
import java.util.Vector;
import java.util.Hashtable;

public class Environment extends Hashtable {
    
  private static final String[]
    PLANT_EDIBILITY = {"all tender", "all tough", "random"},
    CRITTER_EDIBILITY = {"not edible", "edible"},
    ROCK_PLACEMENT = {"random", "user controlled"},
    PLANT_PLACEMENT = {"random", "in fertile cells"};
    
  public static Environment BASIC;
    
  private UI ui;

  Environment() {
      super();
  } // constructor Environment()
  
  Environment(UI ui) {
      this();
      this.ui = ui;
      init();
  } // constructor Environment(UI)
  
  // Access methods.
  
  public int getValue(String key) {
      return ((Param)(get(key))).get();
  }
  
  private int getIntDefault(String key) {
      return getBasic(key).get();
  }
  
  private String getLabel(String key) {
      return getBasic(key).getLabel();
  }
  
  private Param getBasic(String key) {
      return (Param)(BASIC.get(key));
  }
  
//  public Enumeration getBasicAll() {
//      BASIC.elements();
//  }
  
  // Setter methods.
  
  public void init() {
      put("nPlants", new Param(ui, 0, 0, 200, "number of plants"));
      put("plantPlacement", new Param(ui, 0, "plant placement", PLANT_PLACEMENT));
      put("nRocks", new Param(ui, 0, 0, 200, "number of rocks"));
      put("rockPlacement", new Param(ui, 0, "rock placement", ROCK_PLACEMENT));
      put("plantEdibility", new Param(ui, 0, "plant edibility", PLANT_EDIBILITY));
      put("critterEdibility", new Param(ui, 0, "critter edibility", CRITTER_EDIBILITY));
      put("eatingCost", new Param(ui, -3, -50, 0, "cost of eating"));
      put("hittingCost", new Param(ui, -3, -50, 0, "cost of hitting"));
      put("turningCost", new Param(ui, -1, -10, 0, "cost of turning"));
      put("movingCost", new Param(ui, -1, -10, 0, "cost of moving"));
      put("livingCost", new Param(ui, -1, -10, 0, "cost of living"));
      put("chewedPunishment", new Param(ui, -25, -50, 0, "punishment for being chewed"));
      put("collidingPunishment", new Param(ui, -3, -50, 0, "punishment for colliding"));
      put("plantEatingReward", new Param(ui, 25, 0, 200, "reward for eating plant"));
      put("critterEatingReward", new Param(ui, 25, 0, 200, "reward for chewing critter"));
      put("nColumns", new Param(ui, 17, 1, 40, "columns in world"));
      put("nRows", new Param(ui, 15, 1, 36, "rows in world"));
  } // init()
  
} // class Environment
