/**
* A class with static methods for Pixels.
*/

import java.io.*;
import java.awt.Color;
import java.awt.Graphics;

public class Pixel {

public static final Color
  COLOR = Color.white;
  
public static final int
  DOT_RADIUS = 1,
  DOT_DIAMETER = 2;

public static void paint(int[] coords, Graphics g) {
    paint(coords, g, COLOR);
} // paint(int[], Graphics)

public static void paint(int[] coords, Graphics g, Color color) {
    g.setColor(color);
    g.fillOval(coords[0] - DOT_RADIUS, coords[1] - DOT_RADIUS,
	       DOT_DIAMETER, DOT_DIAMETER);
} // paint(int[], Graphics, Color)

public static Thing getThing(int[] coords, WorldCanvas canvas) {
    Cell cell = canvas.getSelectedCell(coords);
    if (cell != null)
	return cell.getContents();
    else
	return null;
} // getThing(int[])

public static int getColorInt(int[] coords, WorldCanvas canvas) {
    Cell cell = canvas.getSelectedCell(coords);
    if (cell != null)
	return (cell.getColorInt());
    else
	return -1;
} // method getColorInt(int[], WorldCanvas)

public static Color getColor(int[] coords, WorldCanvas canvas) {
    Cell cell = canvas.getSelectedCell(coords);
    if (cell != null)
	return (cell.getColor());
    else
	return COLOR;
} // method getColor(int[], WorldCanvas)

public static int getDistance(int x0, int y0, int[] coords) {
    return getDistance(x0, y0, coords[0], coords[1]);
} // method getDistance(int, int, int[])

public static int getDistance(int[] coords1, int[] coords2) {
    return getDistance(coords1[0], coords1[1], coords2[0], coords2[1]);
} // method getDistance(int[], int[])

public static int getDistance(int x0, int y0, int x1, int y1) {
    double xDiff = x1 - x0,
           yDiff = y1 - y0;

    return (int)(Math.sqrt(xDiff * xDiff + yDiff * yDiff));
} // method getDistance(int, int, int[])

} // class Pixel
