/** 
 * SALSA 1.0
 * June 2000
 * Mike Gasser (gasser@indiana.edu)
 */

import java.io.*;
import java.awt.Color;

/** A Thing occupying one Cell that can be eaten by
* a Critter.
*/

public class Plant extends Thing {

  public final static Color
    COLOR = Color.green,
    COLOR2 = Cell.LIGHT_GREEN;

  /**
  * Does the plant need to be hit to be edible?
  */
  boolean
    tough = false,
    mustTouchCrit = false;
 
  Plant(World world, UI ui) {
    super(world, ui, COLOR);
    setToughness(Params.getWorld(Params.PLANT_EDIBILITY) == 1 ||
		 (!(Params.getWorld(Params.PLANT_EDIBILITY) > 1) &&
		 Math.random() < .5));
		 
    mustTouchCrit = (Params.getWorld(Params.PLANT_EDIBILITY) == 3);

    edible = true;

    typeString = "Plant"; 
    world.addPlant (this);
    name = typeString + index;
  } // constructor Plant(World, UI)
  
  /**
  * Writes the state of the Plant to a file.
  */
  /*
  public void write(PrintWriter out) {
    Cell cell = getCell(0);

    out.println("plant");
    out.println("  cell " + cell.getColumn() + " " + cell.getRow());
  }*/ // method write

  public void describe() {
    ui.write("  Plant " + index + ":");
    getCell(0).describe();
  } // method describe

  public boolean isEdible() {
      if (mustTouchCrit)
	  return nextToCritters();
      else
          return (edible && !tough);
  } // method isEdible()
  
  public boolean isTough() {
      return tough;
  } // method isTough()
  
  public void setToughness(boolean value) {
      tough = value;
      if (value)
	  color = COLOR;
      else
	  color = COLOR2;
  } // method setToughness(boolean)
  
  /**
  * Do at least 2 Critters touch the Plant?
  */
  private boolean nextToCritters() {
      Cell[] neighbors = getCell(0).getNeighbors();
      Cell c;
      Thing t;
      int i = 0;
      int critsFound = 0;
      
      while (i < 6 && critsFound < 2) {
	  c = neighbors[i];
	  if (c != null) {
	      t = c.getContents();
	      if (t instanceof Critter)
		  critsFound++;
	  }
	  i++;
      }
      return (critsFound >= 2);
  } // nextToCritters()
  
  public void getHit() {
      tough = false;
      color = COLOR2;
      if (world.frame.isShowing())
        paint();
  } // method getHit()

} // class Plant

