/** 
 * SALSA 1.1
 * June 2000
 * Mike Gasser (gasser@indiana.edu)
 */

import java.awt.*;
import java.awt.event.*;

public class ReinfCanvas extends CritterCanvas {
    
  private final static Font
    FONT2 = new Font("SansSerif", Font.PLAIN, 9);
    
  private static final int
    NAMES_WIDTH = 13,
    WIDTH = 300,
    LEGEND_HEIGHT = 42,
    HEIGHT = 264;
    
  private final static String[]
    LABELS = {"Punishments", "Rewards"},
    PUNISHMENT_LABELS = {"Move", "Turn", "Eat", "Hit", "Step",
                         "Bumped", "Chewed"},
    REWARD_LABELS = {"Plant Eating", "Critter Eating"};
    
  private static final Color[]
    REWARD_COLORS = {Color.green, Color.blue},
    PUNISHMENT_COLORS = {Color.red, Cell.LIGHT_GREEN,
                         Color.magenta, Color.cyan,
			 Color.black, Cell.LIGHT_BLUE,
			 Color.orange};

  private static final Color
    MIDPOINT_COLOR = Color.gray;

  private double
    maximum;
    
  protected int
    legendLabelX0,
    legendLabelX1,
    legendBarX,
    legendBarWidth,
    legendBarHeight,
    
    legendLabelY,
    
    midX,
    barHeight;

  private Strength[] strengths;

  ReinfCanvas(UI ui, World world) {
      super(ui, world);
  } // constructor ReinfCanvas(UI, World)
  
  protected void init() {
      strengths = getStrengths();
      setMax();
  } // init()

  private void setMax() {
      int punishment = Params.getWorstActionPunishment() +
                       Params.getWorstInteractionPunishment() +
		       Params.getWorld(Params.STEP_DECR);
      int reward = Params.getBestReward();
//      ui.write("Worst punishment: " + punishment);
//      ui.write("Best reward: " + reward);
      
      maximum = (double) Math.max(reward, Math.abs(punishment));
//      ui.write("Maximum: " + maximum);
  } // setMax()
 
  protected void setDimensions() {
      int i, y;
      
      mainWidth = WIDTH;
      namesWidth = NAMES_WIDTH;
      width = mainWidth + namesWidth;

      cellHeight = HEIGHT / nCritters;
      mainHeight = cellHeight * nCritters;
      height = mainHeight + LEGEND_HEIGHT + 1;
      barHeight = cellHeight / 2;
      midX = mainWidth / 2;
      
      legendLabelX0 = 2;
      legendLabelX1 = midX + 2;
      legendBarWidth = 23;
      
      legendBarHeight = 4;
      
      legendLabelY = mainHeight + 14;
  } // setDimensions()
  
  protected void drawLines(Graphics g) {
    g.setColor(MIDPOINT_COLOR);
    g.drawLine(midX, 0, midX, height - 1);
    
    g.setColor(BORDER_COLOR);
    g.drawRect(0, 0, width - 1, height - 1);
    g.drawLine(mainWidth, 0, mainWidth, height - 1);
  } // drawLines(Graphics)
    
  protected void writeLegend(Graphics g) {
      // Punishment legend
      g.setColor(Color.black);
      g.setFont(FONT);
      
      g.drawString(LABELS[0], legendLabelX0, legendLabelY);
      
      g.setColor(PUNISHMENT_COLORS[5]);
      g.fillRect(legendLabelX0 + 70, legendLabelY - 8 - legendBarHeight / 2,
 		 legendBarWidth, legendBarHeight);
      g.setColor(PUNISHMENT_COLORS[6]);
      g.fillRect(legendLabelX0 + 70 + legendBarWidth + 18,
		 legendLabelY - 8 - legendBarHeight / 2, legendBarWidth, legendBarHeight);
      
      g.setColor(Color.black);
      g.setFont(FONT2);
      g.drawString(PUNISHMENT_LABELS[5], legendLabelX0 + 70,
		   legendLabelY + 4);
      g.drawString(PUNISHMENT_LABELS[6], legendLabelX0 + 70 + legendBarWidth + 18,
		   legendLabelY + 4);

      g.setColor(PUNISHMENT_COLORS[0]);
      g.fillRect(legendLabelX0 + 2,
		 legendLabelY + 12 - legendBarHeight / 2,
 		 legendBarWidth, legendBarHeight);
      g.setColor(PUNISHMENT_COLORS[1]);
      g.fillRect(legendLabelX0 + 2 + legendBarWidth + 6,
		 legendLabelY + 12 - legendBarHeight / 2,
 		 legendBarWidth, legendBarHeight);
      g.setColor(PUNISHMENT_COLORS[2]);
      g.fillRect(legendLabelX0 + 2 + 2 * (legendBarWidth + 6),
		 legendLabelY + 12 - legendBarHeight / 2,
 		 legendBarWidth, legendBarHeight);
      g.setColor(PUNISHMENT_COLORS[3]);
      g.fillRect(legendLabelX0 + 2 + 3 * (legendBarWidth + 6),
		 legendLabelY + 12 - legendBarHeight / 2,
 		 legendBarWidth, legendBarHeight);
      g.setColor(PUNISHMENT_COLORS[4]);
      g.fillRect(legendLabelX0 + 2 + 4 * (legendBarWidth + 6),
		 legendLabelY + 12 - legendBarHeight / 2,
 		 legendBarWidth, legendBarHeight);

      g.setColor(Color.black);
      g.setFont(FONT2);
      g.drawString(PUNISHMENT_LABELS[0], legendLabelX0 + 2,
		   legendLabelY + 24);
      g.drawString(PUNISHMENT_LABELS[1], legendLabelX0 + 2 + legendBarWidth + 6,
		   legendLabelY + 24);
      g.drawString(PUNISHMENT_LABELS[2],
		   legendLabelX0 + 2 + 2 * (legendBarWidth + 6),
		   legendLabelY + 24);
      g.drawString(PUNISHMENT_LABELS[3],
		   legendLabelX0 + 2 + 3 * (legendBarWidth + 6),
		   legendLabelY + 24);
      g.drawString(PUNISHMENT_LABELS[4],
		   legendLabelX0 + 2 + 4 * (legendBarWidth + 6),
		   legendLabelY + 24);
      
      // Reward legend
      g.setColor(Color.black);
      g.setFont(FONT);
      g.drawString(LABELS[1], legendLabelX1, legendLabelY);

      g.setColor(REWARD_COLORS[0]);
      g.fillRect(legendLabelX1 + 2,
		 legendLabelY + 12 - legendBarHeight / 2,
 		 legendBarWidth, legendBarHeight);
      g.setColor(REWARD_COLORS[1]);
      g.fillRect(legendLabelX1 + 2 + 2 * (legendBarWidth + 6) + 8,
		 legendLabelY + 12 - legendBarHeight / 2,
 		 legendBarWidth, legendBarHeight);

      g.setColor(Color.black);
      g.setFont(FONT2);
      g.drawString(REWARD_LABELS[0], legendLabelX1 + 2,
		   legendLabelY + 24);
      g.drawString(REWARD_LABELS[1],
		   legendLabelX1 + 2 + 2 * (legendBarWidth + 6) + 8,
		   legendLabelY + 24);
  } // writeLegend(Graphics)

  /**
  * Clear the canvas and draw the unit boundaries.
  */
  /*
  public void clear() {
    int critIndex;
    Graphics g = getGraphics();
      
    g.clearRect(0, 0, width, height);
    g.setColor(BORDER_COLOR);
    
    for (critIndex = 0; critIndex < nCritters; critIndex++) {      
      g.drawRect (0, 2 * critIndex * cellHeight, width, 2 * cellHeight);
    }
  }*/ // clear()
  
  public void paintCritter(Graphics g, int critIndex) {
      if (!isDead(critIndex)) {
	  
	  int y = critIndex * cellHeight;
	  Strength s = strengths[critIndex];
	  
	  if (s.dead) {
	      dead[critIndex] = true;
	      g.setColor(DEAD_COLOR);
	      g.fillRect(1, y + 1, width - 2, cellHeight - 1);
	      
	  } else {
	      // Clear out the Critter's strip
	      g.setColor(BACKGROUND_COLOR);
	      g.fillRect(1, y + 1, midX - 1, cellHeight - 1);
	      g.fillRect(midX + 1, y + 1, midX - 1, cellHeight - 1);
	      
	      paintBar(g, s, y + cellHeight / 2);
	  }
      }
  } // paintCritter(Graphics, int)
  
  private void paintBar(Graphics g, Strength s, int y) {
      int p, r, x, w;
      double value;
      
      for (p = Strength.N_PUNISHMENTS - 1, x = midX; p >= 0; p--) {
	  value = (s.getRunPunishment(p) / (s.getTStep() * 2.0 * maximum));
	  w = (int)((width - 2) * -value);
	  if ((x - w) < 1)
	      w = x - 1;
	  g.setColor(PUNISHMENT_COLORS[p]);
	  g.fillRect(x - w, y - barHeight / 2, w, barHeight);
	  x -= w;
      }
      for (r = 0, x = midX + 1; r < Strength.N_REWARDS; r++) {
	  value = (s.getRunReward(r) / (s.getTStep() * 2.0 * maximum));
	  w = (int)((width - 2) * value);
	  if ((x + w) > (width - 1))
	      w = width - 1 - x;
	  g.setColor(REWARD_COLORS[r]);
	  g.fillRect(x, y - barHeight / 2, w, barHeight);
	  x += w;
      }
  } // paintBar(Graphics, Strength, int)
  
} // class ReinfCanvas
