/** 
 * SALSA 2
 * July 2000
 * Mike Gasser (gasser@indiana.edu)
 */

import java.awt.*;
import java.awt.event.*;

public class StatsCanvas extends CritterCanvas {
    
  private static final int
    NAMES_WIDTH = 13,
    WIDTH = 300,
    LEGEND_HEIGHT = 20,
    HEIGHT = 264;
    
  private final static String[]
    LABELS = {"Strength", "Age"};
    
  private static final Color[]
    BAR_COLORS = {Color.green, Color.magenta};

  private static final Color
    MIDPOINT_COLOR = Color.gray;

  private static final int
    STRENGTH = 0,
    AGE = 1;
    
  private static final double[]
    MAXIMUMS = {(double)(Strength.MAX), 10000.0};
    
  protected int
    legendLabelX0,
    legendLabelX1,
    legendLabelY,
    legendBarX,
    legendBarWidth,
    
    midX,
    barHeight;

  public int
    tStep = 1;
    
  private Strength[] strengths;
  
  private char[] names;

  private boolean[]
    dead;
  
  StatsCanvas(UI ui, World world) {
      super(ui, world);
  } // constructor StatsCanvas(UI, World)
  
  protected void init() {
      strengths = getStrengths();
  }

  protected void setDimensions() {
      int i, y;
      
      mainWidth = WIDTH;
      namesWidth = NAMES_WIDTH;
      width = mainWidth + namesWidth;
      cellHeight = HEIGHT / nCritters;
      mainHeight = cellHeight * nCritters;
      height = mainHeight + LEGEND_HEIGHT + 1;
      barHeight = cellHeight / 2;
      midX = mainWidth / 2;
      
      legendLabelX0 = 2;
      legendLabelX1 = midX + 2;
      legendLabelY = mainHeight + 14;
      legendBarWidth = 30;
  } // setDimensions()
 
  protected void drawLines(Graphics g) {
    g.setColor(MIDPOINT_COLOR);
    g.drawLine(midX, 0, midX, height - 1);
    
    g.setColor(BORDER_COLOR);
    g.drawRect(0, 0, width - 1, height - 1);
    g.drawLine(mainWidth, 0, mainWidth, height - 1);
  } // drawLines(Graphics)
    
  protected void writeLegend(Graphics g) {
      g.setColor(Color.black);
      g.setFont(FONT);
      
      g.drawString(LABELS[0], legendLabelX0, legendLabelY);
      g.drawString(LABELS[1], legendLabelX1, legendLabelY);
  } // writeLegend(Graphics)

  public void paintCritter(Graphics g, int critIndex) {
      if (!isDead(critIndex)) {
	  
	  int y = critIndex * cellHeight;
	  Strength s = strengths[critIndex];

	  if (s.dead) {
	      dead[critIndex] = true;
	      g.setColor(DEAD_COLOR);
	      g.fillRect(1, y + 1, width - 2, cellHeight - 1);
	      
	  } else {
	      // Clear out the Critter's strip
	      g.setColor(BACKGROUND_COLOR);
	      g.fillRect(1, y + 1, midX - 1, cellHeight - 1);
	      g.fillRect(midX + 1, y + 1, midX - 2, cellHeight - 1);
	      
	      y += cellHeight / 2;
	      paintBar(g, s, STRENGTH, 1, y);
	      paintBar(g, s, AGE, midX + 1, y);
	  }
      }
  } // paintCritter(Graphics, int)
  
  private void paintBar(Graphics g, Strength s, int barIndex, int x, int y) {
      double value = getBarWidth(s, barIndex);

      g.setColor(BAR_COLORS[barIndex]);
//      ui.write("Bar value (" + barIndex + "): " + value);
      value = Math.min(1.0, Math.abs(value));

      g.fillRect(x,
		 y - barHeight / 2,
		 (int)((mainWidth - 2) * value),
		 barHeight);
  } // paintBar(Graphics, Strength, int)
  
  private double getBarWidth(Strength s, int barIndex) {
      if (barIndex == STRENGTH)
	  return (s.getValue() / (MAXIMUMS[STRENGTH] * 2.0));
      else
	  return (s.getAge() / (MAXIMUMS[AGE] * 2.0));
  } // getBarWidth(Strength, int)

} // class StatsCanvas
