/** 
 * SALSA 1.1
 * June 2000
 * Mike Gasser (gasser@indiana.edu)
 */

import java.io.*;
import java.awt.Color;

public class Strength {

  public static final int
    // Negative
    MOVE = 0,
    TURN = 1,
    EAT = 2,
    HIT = 3,
    STEP = 4,
    COLLISION = 5,
    EATEN = 6,

    // Positive
    PLANT_EATING = 0,
    CRITTER_EATING = 1,

    N_PUNISHMENTS = 7,
    N_REWARDS = 2;
    
  public static final int
    INIT_GAP = 100,
    MAX = 2000;
    
  private UI ui;
  
  private int value = MAX - INIT_GAP;
  
  private int reinforcement = 0;
  
  private int runReinforcement = 0;
  
  private int reincarnations = 0;

  private int[]
    runPunishments = new int[N_PUNISHMENTS],
    runRewards = new int[N_REWARDS];
    
  private int age = 0;
  
  // Age since the critter was created.
  private int realAge = 0;
  
  // Time steps in the current run.
  private int tStep = 0;
  
  public boolean dead = false;
    
  /**
  * This constructor just sets the UI.
  */
  Strength(UI ui, int age) {
    this.ui = ui;
    this.age = age;
  } // constructor Strength

  // Access methods

  public int getValue() {
      return value;
  } // getValue()
  
  public int getAge() {
      return age;
  } // getAge()
  
  public int getRealAge() {
      return realAge;
  } // getRealAge()
  
  public int getReinforcement() {
      return reinforcement;
  } // getReinforcement()
  
  public int getRunReinforcement() {
      return runReinforcement;
  } // getRunReinforcement()
  
  public int getRunReward(int type) {
      return runRewards[type];
  } // getRunReward(int)
  
  public int getRunPunishment(int type) {
      return runPunishments[type];
  } // getRunPunishment(int)
  
  public int getTStep() {
      return tStep;
  } // getTStep()
  
  public int getNReincarnations() {
      return reincarnations;
  } // getNReincarnations()
  
  // Setter methods
  
  public void setValue(int v) {
      value = v;
  } // setValue(int)
  
  private void addReinforcement(int a, int type) {
//      ui.write("Adding reinforcement: " + a);
      reinforcement += a;
      runReinforcement += a;
      if (a > 0)
	  runRewards[type] += a;
      else
	  runPunishments[type] += a;
  } // addReinforcement(int, int)
  
  public void incrAge() {
      age++;
      realAge++;
  } // incrAge()
  
  public void step() {
      tStep++;
  } // step()
  
  public void reinit() {
      value = MAX - INIT_GAP;
      age = 0;
      realAge = 0;
      dead = false;
      reincarnations++;
      initRun();
  } // reinit()
  
  public void initStep() {
      reinforcement = 0;
  } // initStep()
  
  public void initRun() {
      int i;
//      initStep();
      tStep = 0;
      runReinforcement = 0;
      for (i = 0; i < N_REWARDS; i++)
	  runRewards[i] = 0;
      for (i = 0; i < N_PUNISHMENTS; i++)
	  runPunishments[i] = 0;
  } // initRun()
  
  public void add(int a, int type) {
      int newV = value + a;
//      ui.write("Adding reinf: " + a);

      if (newV < 0) {
	  addReinforcement(-value, type);
	  value = 0;
	  
      } else if (newV > MAX) {
	  addReinforcement(MAX - value, type);
	  value = MAX;

      } else {
	addReinforcement(a, type);
	value = newV;
      }
  } // add(int, int)
  
  public void describe() {
      ui.write("  Strength: current " + value + ", reinf for run " +
	       (runReinforcement / UI.nRunTimeSteps));
  } // describe()

 /**
  * Converts a strength to a shade of the original color
  * (lighter means less strength).  RGB are the initial RGB int values.
  */
  protected static Color toColor(int s, int[] RGB) {
      int
	strengthDiff = MAX - s,
	red = RGB[0],
	green = RGB[1],
	blue = RGB[2];

       return (new Color (red + ((255 - red) * strengthDiff) / MAX,
			  green + ((255 - green) * strengthDiff) / MAX,
			  blue + ((255 - blue) * strengthDiff) / MAX));
  } // toColor(int, int[])
  
} // class Strength
