/** 
 * SALSA 2
 * July 2000
 * Mike Gasser (gasser@indiana.edu)
 */

import java.io.*;
import java.util.Vector;
import java.util.Hashtable;
import java.awt.Color;

public class Params {

  // PUBLIC CLASS CONSTANTS
  
  // Directory where data are stored.
  public final static String
    DIRECTORY = "Data",
    FILE = "setup";
    
  public final static String[]
    SENSOR_LABELS = {"Touch", "Vision", "Sonar"},
    EFFECTOR_LABELS = {"Rest", "Eat", "Move", "Hit", "Turn"};
    
  private final static String[]
    DISPLAY_KEYS =
    {"showReinforcement",
     "showSTM",
     "showLTM",
     "showStrengths"},

    WORLD_KEYS =
    {"nPlants", 
     "plantPlacement",
     "nRocks",
     "barrier",
     "rockPlacement",
     
     "critterEdibility",
     "plantEdibility",
     
     "eatingDecr",
     "turnDecr",
     "moveDecr",
     "stepDecr",
     "hittingDecr",
     
     "eatenDecr",
     "collisionDecr",
     
     "plantEatingIncr",
     "critterEatingIncr",
     
     "worldColumns",
     "worldRows"},
     
     SPECIES_KEYS =
     {"touchRange",
      
      "nColors",
      "visualAngle",
      "visualResolution",
      
      "sonarResolution",
      "nSonarDistinctions",
      "sonarAngle",
      
      "rest",
      "turn",
      "move",
      "eat",
      "hit",
      
      "learningRate",
      "discountRate",
      
      "exploitationRate",
      
      "N",
      "name"};
    
  public final static Color[]
    COLORS = {Color.black, Color.red, Cell.LIGHT_GREEN, Color.green, Cell.MEDIUM_BLUE};
    
  public final static int[]
    FEELS = {Cell.EMPTY, Cell.SQUISHY, Cell.SOFT, Cell.ROUGH, Cell.HARD};
    
  // Indeces in various arrays.
  public final static int
    NSONAR = 4,

    NDISPLAY_PARAMS = 4,
    NWORLD_PARAMS = 18,
    // Two greater than number of params to have room for number in
    // in species and name.
    NCRITTER_PARAMS = 17,
    
    SHOW_REINFORCEMENT = 0,
    SHOW_STM = 1,
    SHOW_LTM = 2,
    SHOW_STRENGTHS = 3,

    NSENSORS = 3,
    NEFFECTORS = 5,
    
    TOUCH = 0,
    VISION = 1,
    SONAR = 2,

    // Plant placement
    RANDOM = 0,
    NEXTTO_ROCK = 1,
    AWAY_FROM_ROCKS = 2,
    USER_CONSTRAINED = 3,
    
    // Rock placement
    USER_SPECIFIED = 1,
    
    // Plant edibility
    TOUGH = 1,
    TENDER = 2,
    NEXTTO_CRITTER = 3,            // Not currently used.
    
    // Critter edibility
    INEDIBLE = 0,
    EDIBLE = 1,

    // Order of actions
    REST = 0,
    EAT = 1,
    MOVE = 2,
    HIT = 3,
    TURN = 4,
    
    // Sensors
    TOUCH_RANGE = 0,

    NCOLORS = 1,
    VISUAL_ANGLE = 2,
    VISUAL_RESOLUTION = 3,
    
    SONAR_RESOLUTION = 4,
    NSONAR_DISTINCTIONS = 5,
    SONAR_ANGLE = 6,
    
    // Effectors
    NREST = 7,
    NTURN = 8,
    NMOVE = 9,
    NEAT = 10,
    NHIT = 11,
    
    // Learning parameters
    LEARNING_RATE = 12,
    DISCOUNT_RATE = 13,

    // Action parameter
    EXPLOITATION_RATE = 14,
    
    // Other species parameters
    N = 15,
    NAME = 16,
    
    // Environment parameters
    // Things
    NPLANTS = 0,
    PLANT_PLACEMENT = 1,
    NROCKS = 2,
    BARRIER = 3,
    ROCK_PLACEMENT = 4,
    
    // Edibility
    CRITTER_EDIBILITY = 5, 
    PLANT_EDIBILITY = 6,
    
    // Reinforcement
    EATING_DECR = 7,
    TURN_DECR = 8,
    MOVE_DECR = 9,
    STEP_DECR = 10,
    HITTING_DECR = 11,

    EATEN_DECR = 12,
    COLLISION_DECR = 13,
    
    PLANT_EATING_INCR = 14,
    CRITTER_EATING_INCR = 15,

    // World parameters
    // Dimensions
    NCOLUMNS = 16,
    NROWS = 17;
    
  // String constants needed for reading in World parameters, Things,
  // and Thing parameters.  Used in read.
  public final static String
    DISPLAY_KEY = "display",
    ENVIRONMENT_KEY = "environment",
    ROCK_KEY = "rock",
    FERTILITY_KEY = "fertile",
    // Species
    CRITTERS_KEY = "critters",
    AGE_KEY = "age",
    MEMORY_KEY = "memory";
    
  public static final int
    VALUE = 0,
    MINIMUM = 1,
    MAXIMUM = 2;
    
  // PUBLIC CLASS VARIABLES
    
  public static int[]
    DISPLAY = {1, 1, 1, 1};
    
  public static int[][]
    WORLD = new int[NWORLD_PARAMS][3];
    
  public static Vector
//    SEASONS = new Vector(),
    SPECIES = new Vector();
    
  public static Vector
    // Coordinates of saved rocks and fertile places.
    rockPlaces = new Vector(),
    fertilePlaces = new Vector();
    
  private static Color[]
    colors;
    
  private static int[]
    feels;
    
  public static String
    envFilename = null;
    
  public static boolean
    // Whether randomly placed Things are allowed to touch
    // others.
    thingsCanAbut = true;

  public static int
    minCellDistance = 0,
    maxCellDistance = 100;

  public static int
    // Ms between animation steps.
    animationPause = 50;

  // Keeps track of number of things so it doesn't get too big.
  public static int
    nThings = 0,
    nCritters = 0;

  public static int getWorld(int index) {
      return WORLD[index][VALUE];
  }
  
  public static int getDisplay(int index) {
      return DISPLAY[index];
  }
  
  public static int getNSpecies() {
      return SPECIES.size();
  } // getNSpecies()
  
  /**
  * Given a seoson index and a parameter index, returns the
  * value for that season.   
  */
  /*
  public static int getWorld(int season, int index) {
      return getSeason(season)[index];
  }*/ // getWorld(int, int)
    
  public static int getWorldMin(int index) {
      return WORLD[index][MINIMUM];
  }
  
  public static int getWorldMax(int index) {
      return WORLD[index][MAXIMUM];
  }

  public static Species getSpecies(int i) {
      return (Species)(SPECIES.elementAt(i));
  } // getSpecies(int)
  
  public static String getWorldKey(int i) {
      return WORLD_KEYS[i];
  } // getWorldKey(int)
  
  public static String getSpeciesKey(int i) {
      return SPECIES_KEYS[i];
  } // getSpeciesKey(int)
  
  public static String getDisplayKey(int i) {
      return DISPLAY_KEYS[i];
  }
  
  public static int getWorstActionPunishment() {
      int worst = 0;
      int val;
      val = getWorld(EATING_DECR);
      if (val < worst)
	  worst = val;
      val = getWorld(HITTING_DECR);
      if (val < worst)
	  worst = val;
      val = 3 * getWorld(MOVE_DECR);
      if (val < worst)
	  worst = val;
      val = 3 * getWorld(TURN_DECR);
      if (val < worst)
	  worst = val;
      
      return worst;
  } // getWorstActionPunishment()
  
  public static int getWorstInteractionPunishment() {
      int worst = 0;
      int val;
      
      worst += getWorld(COLLISION_DECR);

      if (getWorld(CRITTER_EDIBILITY) > 0) {
	  worst += 2 * getWorld(EATEN_DECR);
      }
      
      return worst;
  } // getWorstInteractionPunishment()
  
  public static int getBestReward() {
      int best = 0;
      int val;
      if (getWorld(NPLANTS) > 0) {
        val = getWorld(PLANT_EATING_INCR);
        if (val > best)
	  best = val;
      }
      
      if (getWorld(CRITTER_EDIBILITY) > 0) {
          val = getWorld(CRITTER_EATING_INCR);
          if (val > best)
	      best = val;
      }
      return best;
  } // getBestReward()

  /*
  public static int[] getSeason(int i) {
      return (int [])(SEASONS.elementAt(i));
  }*/ // getSeason(int)
  
  /**
  * Minimum and maximum number of possible Critters.  Later
  * fix these.   
  */
  private static int getNCritMin() {
      return 1;
  } // getNCritMin()
  
  private static int getNCritMax() {
      return 25;
  } // getNCritMax()
  
  public static Color getColor(int i) {
      return COLORS[i];
  } // getColor(int)
  
  /*
  public static int getNColors() {
      return colors.length;
  }*/ // getNColors()
  
  public static int getFeel(int i) {
      return FEELS[i];
  } // getFeel(int)
  
  /*
  public static int getNFeels() {
      return feels.length;
  } */ // getNFeels()

  public static String getSpeciesNames() {
      String names = "";
      for (int s = 0; s < getNSpecies(); s++)
	  names += getSpecies(s).getName();
      return names;
  } // getSpeciesNames()
  
  // SETTERS
  
  public static void setWorld(int index, int slot, int value) {
      WORLD[index][slot] = value;
  } // setWorld(int, int, int)
  
  private static void setCrit(int[] values, int index, int value) {
      values[index] = value;
  } // setCrit(int[][], int, int, int)
  
  private static void setWorld(int index, int def, int min, int max) {
      setWorld(index, VALUE, def);
      setWorld(index, MAXIMUM, max);
      setWorld(index, MINIMUM, min);
  } // setWorld(int, int, int, int)
  
  public static void setDisplay(int index, int value) {
      DISPLAY[index] = value;
  } // setDisplay(int, int)
  
  public static void set() {
      // Things in the World.
      setWorld(NPLANTS, 0, 0, 200);
      setWorld(PLANT_PLACEMENT, 0, 0, 3);
      setWorld(NROCKS, 0, 0, 200);
      setWorld(BARRIER, 0, 0, 10);
      setWorld(ROCK_PLACEMENT, 0, 0, 1);
      
      // Edibility
      setWorld(PLANT_EDIBILITY, 0, 0, 3);
      setWorld(CRITTER_EDIBILITY, 0, 0, 1);

      // Reinforcement
      setWorld(COLLISION_DECR, -3, -50, 0);
      setWorld(EATING_DECR, -3, -50, 0);
      setWorld(HITTING_DECR, -3, -50, 0);
      setWorld(EATEN_DECR, -25, -50, 0);
      
      setWorld(TURN_DECR, -1, -10, 0);
      setWorld(MOVE_DECR, -1, -10, 0);
      setWorld(STEP_DECR, -1, -10, 0);
      
      setWorld(PLANT_EATING_INCR, 25, 0, 200);
      setWorld(CRITTER_EATING_INCR, 25, 0, 200);
      
      // World size
      setWorld(NCOLUMNS, 17, 1, 40);
      setWorld(NROWS, 15, 1, 36);
  } // set()
  
  public static Species addCritSpecies(UI ui, int[] values, Memory mem, int age) {
      Species spec = new Species(ui, values);
      SPECIES.addElement(spec);
      
      if (mem != null) {
	  spec.setMemory(mem);
	  mem.setSpecies(SPECIES.size() - 1);
	  mem.setParams();
      }
      if (age > 0)
	  spec.setAge(age);
      
      nThings += values[N];
      nCritters += values[N];
      
      return spec;
  } // addCritSpecies(int[])
  
  private static boolean set(UI ui, int[] values, int index, int value, boolean critter) {
      int[][] minmax = (critter ? Species.VALUES : WORLD);
      
      if (value < minmax[index][MINIMUM]) {
          ui.write(value + " is too small!");
          return false;
      } else if (value > minmax[index][MAXIMUM]) {
	  ui.write(value + " is too large!");
	  return false;
      } else {
	  values[index] = value;
	  return true;
      }
  } // set(int[], int, int)
  
  private static boolean setWorld(UI ui, int index, int value) {
      if (value < getWorldMin(index)) {
          ui.write(value + " is too small!");
          return false;
      } else if (value > getWorldMax(index)) {
	  ui.write(value + " is too large!");
	  return false;
      } else {
	  setWorld(index, VALUE, value);
	  return true;
      }
  } // setWorld(UI, int, double)
  
  private static void setSpeciesN(int[] species, int n) {
      species[N] = n;
  } // setSpecies(int[], int)
  
  private static void setSpeciesName(int[] species, char name) {
      species[NAME] = name;
  } // setSpeciesName(int[], char)
  
  /**
  * Creates an int array and fills it with default Critter values.
  */
  private static int[] makeSpeciesParams() {
      int[] species = new int[NCRITTER_PARAMS];
      int i;
      
      for (i = 0; i < NCRITTER_PARAMS; i++) {
          species[i] = Species.getDefault(i);
      }
      return species;
  } // makeSpeciesParams()
  
  private static void abort(UI ui) {
      ui.write("Aborting!");
      reinit();
  } // abort(UI)
  
  private static void initDisplay() {
      DISPLAY[0] = 1;
      DISPLAY[1] = 1;
      DISPLAY[2] = 1;
      DISPLAY[3] = 1;
  } // initDisplay()
  
  public static void reinit() {
      SPECIES.removeAllElements();
      rockPlaces.removeAllElements();
      fertilePlaces.removeAllElements();
      nThings = 0;
      nCritters = 0;
      // Sets values to defaults.
      set();
      initDisplay();
  } // reinit()
  
  private static boolean respondCritter(UI ui, String key, int val, int[] values) {
      boolean found = false;
      int i = 0;
      
      while (!found && i < NCRITTER_PARAMS) {
          if (key.equalsIgnoreCase(getSpeciesKey(i))) {
	      found = true;
          } else
	      i++;
      }

      if (found)
	  return set(ui, values, i, val, true);
      else {
	  ui.write("WARNING: parameter " + key + " not recognized!");
	  return false;
      }
      
  } // respondCritter(UI, String, int, int[])
  
  /*
  protected static boolean respondDisplay (UI ui, String key, int val) {
      boolean found = false;
      int i = 0;
      
      while (!found && i < NDISPLAY_PARAMS) {
          if (key.equalsIgnoreCase(getDisplayKey(i))) {
	      found = true;
          } else
	      i++;
      }

      if (found)
	  setDisplay(i, val);
      else
	  ui.write("Parameter " + key + " not recognized!");

      return found;
      
  }*/ // respondDisplay(UI, String, int)
  
  /**
  * Determines what to do with keywords and data read in
  * from a file.   
  */
  protected static boolean respond (UI ui, String key, int val) {
      boolean found = false;
      int i = 0;
      
      while (!found && i < NWORLD_PARAMS) {
          if (key.equalsIgnoreCase(getWorldKey(i))) {
	      found = true;
          } else
	      i++;
      }

      if (found)
	  return setWorld(ui, i, val);
      else {
	  ui.write("WARNING: parameter " + key + " not recognized!");
	  return false;
      }
      
  } // respond(UI, String, int)
  
 /**
 * Reads in parameters from the params file or an environment file.
 */
 public static boolean read(Tokenizer tok, UI ui, String directory) {
    boolean ok = true;
    
    try {
	 String key = tok.getKeyword();
	 int[] values;
	 while (key != null && ok) {
//	     ui.write("Key: " + key);
	     if (key.equalsIgnoreCase(CRITTERS_KEY)) {
		 ok = readCritSpecies(ui, tok, directory);
		 key = tok.getKeyword();
		
	     } else if (key.equalsIgnoreCase(DISPLAY_KEY)) {
		 ok = readDisplay(ui, tok);
		 key = tok.getKeyword();

	     } else if (key.equalsIgnoreCase(ENVIRONMENT_KEY)) {
		 ok = readEnvironment(ui, tok, directory);
		 key = tok.getKeyword();
		 
	     } else if (key.equalsIgnoreCase(ROCK_KEY)) {
		 int[] coords = new int[2];
//		 ui.write("Reading in rock.");
		 // coordinates of cell
		 tok.nextToken();
		 coords[0] = (int)(tok.nval);
		 tok.nextToken();
		 coords[1] = (int)(tok.nval);
		 rockPlaces.addElement(coords);
		 setWorld(ui, Params.NROCKS, 1);
		 key = tok.getKeyword();
		 
	     } else if (key.equalsIgnoreCase(FERTILITY_KEY)) {
		 int[] coords = new int[2];
//		 ui.write("Reading in fertile cell.");
		 // coordinates of cell
		 tok.nextToken();
		 coords[0] = (int)(tok.nval);
		 tok.nextToken();
		 coords[1] = (int)(tok.nval);
		 fertilePlaces.addElement(coords);
		 key = tok.getKeyword();
		 
	     } else {
	       tok.nextToken();
	       if (tok.ttype != StreamTokenizer.TT_NUMBER) {
	         ui.write(tok.sval + " should be a number!");
		 abort(ui);
  	         ok = false;
	       } else {
  	         ok = respond(ui, key, (int)(tok.nval));
	         key = tok.getKeyword();
	       }
	     }
	 }
     }
     catch (IOException e) {
	 System.err.println("Params: read: " + e);
     }
     
     return ok;
  } // read(Tokenizer, UI, String)
  
  public static boolean read(UI ui, String directory, String fname) {
    String path = directory + fname;
//      System.getProperty("user.dir") + UI.SEPARATOR +
//         DIRECTORY + UI.SEPARATOR + fname;

    boolean ok = true;
        
    File file = new File(path);

    if (file.exists()) {
      try {
         String key;
         Tokenizer tok = new Tokenizer(new FileInputStream(file));

//         ui.write("Reading in parameters.");
	 ok = read(tok, ui, directory);
      }
      catch (IOException e) {
	  System.err.println("Params:read: " + e);
      }
    } else {
      ui.write("No file found.");
      ok = false;
    }
    
//    setColors();
    
    for (int s = 0; s < getNSpecies(); s++) {
        getSpecies(s).init();
        getSpecies(s).describe();
    }
  
    return ok;
  } // method read(UI, String, String)
  
  private static boolean readEnvironment(UI ui, String filename, String directory) {
      boolean ok = true;
      String fname = directory + filename;
//        fname = System.getProperty ("user.dir") + UI.SEPARATOR +
//           DIRECTORY + UI.SEPARATOR + filename;

      File file = new File (fname);
      
      if (file.exists()) {
        try {
           String key;
           Tokenizer tok = new Tokenizer(new FileInputStream(file));
	   ok = read(tok, ui, directory);
	   envFilename = filename;
	   ui.write("Environment " + filename);
        }
        catch (IOException e) {
	  System.err.println("Params:readEnvironment: " + e);
        }
      } else {
        ui.write("File " + filename + " not found.");
	ok = false;
      }
      
      return ok;
  } // readEnvironment(UI, String, String)
  
  private static boolean readEnvironment(UI ui, Tokenizer tok, String directory) {
      boolean ok = true;
      String filename = null;
      
      try {
	  // Must be a filename
	  tok.nextToken();
	  if (tok.ttype == StreamTokenizer.TT_WORD) {
	      filename = tok.sval;
	      ok = readEnvironment(ui, filename, directory);

	  } else {
   	    ui.write(tok.sval + " should be the name of an environment file.");
	    abort(ui);
	    ok = false;
	  }
      }
      catch (IOException e) {
        System.err.println("Params:readEnvironment: " + e);
      }

      return ok;
  } // readEnvironment(UI, Tokenizer, String)
  
  private static boolean readCritSpecies(UI ui, Tokenizer tok, String directory) {
      boolean ok = true;
      String filename = null;
      Species spec = null;
//      int[] values = null;
      
      try {
	  // Either a file name or a number (the species name)
	  tok.nextToken();
	  if (tok.ttype == StreamTokenizer.TT_WORD) {
	      filename = tok.sval;
	      tok.nextToken();
	      if (tok.ttype != StreamTokenizer.TT_NUMBER) {
		  ui.write(tok.sval + " should be the number of critters.");
		  abort(ui);
		  ok = false;
	      } else {
		  spec = readCritSpecies(ui, filename, directory, (int)(tok.nval));
		  if (spec == null)
		      ok = false;
	      }
	  } else if (tok.ttype != StreamTokenizer.TT_NUMBER) {
   	    ui.write(tok.sval + " should be the number of critters.");
	    abort(ui);
	    ok = false;
	  } else {
	      spec = readCritSpecies(ui, tok, (int)(tok.nval), false);
	      if (spec == null)
		  ok = false;
	  }
      }
      catch (IOException e) {
        System.err.println("Params:readCritSpecies: " + e);
      }

      if (spec != null) {
//	  Species spec = addCritSpecies(ui, values, null, 0);
	  if (nCritters > getNCritMax()) {
	      ui.write("Too many critters, maximum is " + getNCritMax());
	      abort(ui);
	      ok = false;
	  }
      }

      return ok;
  } // readCritSpecies(UI, Tokenizer, String)
  
  public static Species readCritSpecies(UI ui, String filename, String directory, int n) {
      String fname = directory + filename;
//      System.getProperty ("user.dir") + UI.SEPARATOR +
//           DIRECTORY + UI.SEPARATOR + filename;
      File file = new File (fname);
      Species spec = null;

      if (file.exists()) {
        try {
           String key;
           Tokenizer tok = new Tokenizer(new FileInputStream(file));
	   spec = readCritSpecies(ui, tok, n, true);
        }
        catch (IOException e) {
	  System.err.println("Params:readCritSpecies: " + e);
        }
      } else {
        ui.write("File " + filename + " not found.");
      }

      return spec;
  } // readCritSpecies(UI, String, String, int)
  
  private static Species readCritSpecies(UI ui, Tokenizer tok, int n, boolean inFile) {
    int[] values = makeSpeciesParams();
    int age = 0;
    Memory mem = null;
    boolean ok = true;
    
    setSpeciesN(values, n);

    try {
	 String key = tok.getKeyword();
	 while (key != null && ok) {
//	     ui.write("Key: " + key);
	     tok.nextToken();

	     if (key.equalsIgnoreCase(getSpeciesKey(NAME))) {
	         ui.write("Reading species name.", 1);
		 if (tok.ttype == StreamTokenizer.TT_WORD)
  	           setSpeciesName(values, tok.sval.charAt(0));
	         else
		   setSpeciesName(values, (char)(tok.nval));
		 key = tok.getKeyword();
		 
	     } else if (key.equalsIgnoreCase(AGE_KEY)) {
		 ui.write("Reading critter age.", 1);
		 age = (int)(tok.nval);
		 key = tok.getKeyword();

	     } else if (key.equalsIgnoreCase(MEMORY_KEY)) {
		 ui.write("Reading critter memory.", 1);
		 tok.pushBack();
		 mem = new Memory(ui);
		 ok = mem.read(tok);
		 key = tok.getKeyword();
		 
	     } else if (tok.ttype == StreamTokenizer.TT_WORD) {
               ui.write(tok.sval + " should be a number!");
	       abort(ui);
	       ok = false;

	     } else {
  	       ok = respondCritter(ui, key, (int)(tok.nval), values);
	       key = tok.getKeyword();
	     }
	 }
    }
     catch (IOException e) {
	 System.err.println("World: readCritSpecies: " + e);
     }
     if (ok) {
	 return addCritSpecies(ui, values, mem, age);
     } else
	 return null;
  } // readCritSpecies(UI, Tokenizer, int, boolean)
  
  private static boolean readDisplay(UI ui, Tokenizer tok) {
    boolean ok = true;
    
    try {
	 String key = tok.getKeyword();
	 boolean found = false;
	 int i = 0;
	 
	 while (key != null && ok) {
//	     ui.write("  Key: " + key);
	     i = 0;
	     found = false;
	     while (!found && i < NDISPLAY_PARAMS) {
		 if (key.equalsIgnoreCase(getDisplayKey(i))) {
		     found = true;
		 } else
		     i++;
	     }
	     if (found) {
//		 ui.write("  Setting " + key);
		 tok.nextToken();
		 setDisplay(i, (int)(tok.nval));
		 key = tok.getKeyword();
	     } else
		 ok = false;
	 }
	 
	 if (key != null) {
	     tok.pushBack();
	     ok = true;
	 }
     }
     catch (IOException e) {
	 System.err.println("World: readDisplay: " + e);
     }
     return ok;
  } // readDisplay(UI, Tokenizer)
  
} // class Params
